@@ -1,18 +1,18 @@
-__version__ = '1.3.0'
+__version__ = '1.3.3'
 # this has to be set here for the test coverage to work
 __name__ = 'nerdvision'
 agent_name = 'nerd.vision Python Agent'
 
 __version_major__ = '1'
 __version_minor__ = '3'
-__version_micro__ = '0'
+__version_micro__ = '3'
 
 __props__ = {
-    '__Git_Branch__': '1.3.0',
-    '__Git_Commit_Id__': 'df6e6e7af785e4e86d4dcac257a20a86dad55506',
-    '__Git_Commit_Time__': '2021-06-16 13:15:24+00:00',
+    '__Git_Branch__': '1.3.3',
+    '__Git_Commit_Id__': 'ce0967c701562f1119370abf26c372438037da32',
+    '__Git_Commit_Time__': '2022-03-18 11:04:56+00:00',
     '__Git_Dirty__': 'False',
-    '__Git_Remote_Origin_Url__': 'https://gitlab-ci-token:AYjzas1RfyP3eGUr3Yxj@gitlab.com/intergral/nerdvision/agents/python-client.git',
+    '__Git_Remote_Origin_Url__': 'https://gitlab-ci-token:iUuyzJ4TFZAtR3KYhf61@gitlab.com/intergral/nerdvision/agents/python-client.git',
 
     '__X_CI_Pipeline_Id__': '',
     '__X_CI_Pipeline_Iid__': '',
@@ -21,13 +21,16 @@ __props__ = {
     '__X_CI_Project_Name__': '',
 }
 
-nv_client = None
-
 try:
     from typing import TYPE_CHECKING as TYPES
 except ImportError:
     TYPES = False
 
+if TYPES:
+    from typing import Optional
+
+nv_client = None  # type: Optional[NerdVisiion]
+
 
 def start(api_key=None, name=None, tags=None, agent_settings=None, serverless=False):
     """
@@ -109,7 +112,7 @@ def configure_logger(force_init=False, serverless=False):
     stream_handler.setFormatter(formatter)
     stream_handler.setLevel(level)
 
-    our_logger.propagate = False
+    our_logger.propagate = True
     our_logger.setLevel(level)
 
     our_logger.addHandler(stream_handler)
